using System;

using CMS.CMSHelper;
using CMS.GlobalHelper;
using CMS.LicenseProvider;
using CMS.SiteProvider;
using CMS.UIControls;
using CMS.WorkflowEngine;
using CMS.SettingsProvider;

public partial class CMSSiteManager_Development_Workflows_Workflow_Scope_Edit : SiteManagerPage
{
    #region "Constants"

    private const string ALL_DOCTYPES = "-1";

    #endregion


    #region "Protected vairables"

    protected int workflowScopeId = 0;
    protected int workflowId = 0;
    protected int siteId = 0;

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.administration", "Modify"))
        {
            RedirectToAccessDenied("cms.administration", "Modify");
        }

        // Set selector mode
        selectClassNames.UniSelector.SelectionMode = SelectionModeEnum.SingleDropDownList;
        selectClassNames.UniSelector.AllowAll = true;

        string startingAliasPath = string.Empty;
        int classId = 0;
        int cultureId = 0;

        workflowScopeId = QueryHelper.GetInteger("scopeid", 0);

        if (workflowScopeId > 0)
        {
            WorkflowScopeInfo wsi = WorkflowScopeInfoProvider.GetWorkflowScopeInfo(workflowScopeId);
            if (wsi != null)
            {
                workflowId = wsi.ScopeWorkflowID;
                siteId = wsi.ScopeSiteID;
                startingAliasPath = wsi.ScopeStartingPath;
                classId = wsi.ScopeClassID;
                cultureId = wsi.ScopeCultureID;
            }
        }
        else
        {
            workflowId = QueryHelper.GetInteger("workflowid", 0);
            siteId = QueryHelper.GetInteger("siteid", 0);
        }

        // Check languages on site
        SiteInfo si = SiteInfoProvider.GetSiteInfo(siteId);
        if (si != null)
        {
            if (!CultureInfoProvider.LicenseVersionCheck(si.DomainName, FeatureEnum.Multilingual, VersionActionEnum.Edit))
            {
                lblError.Text = ResHelper.GetString("licenselimitation.siteculturesexceeded");
                lblError.Visible = true;
                pnlForm.Enabled = false;

            }
            // Set selector's where condition
            selectClassNames.UniSelector.WhereCondition = "ClassID IN (SELECT ClassID FROM CMS_ClassSite WHERE SiteID=" + si.SiteID + ") AND ClassIsDocumentType = 1";
        }

        // Culture selector
        //cultureSelector.DropDownCultures.CssClass = "SelectorDropDown";
        cultureSelector.UseCultureCode = false;
        cultureSelector.SpecialFields = new string[,] { { ResHelper.GetString("general.selectall"), "0" } };
        cultureSelector.SiteID = siteId;

        pathElem.SiteID = siteId;

        // Scripts for path selector
        if (!RequestHelper.IsPostBack())
        {
            pathElem.Value = startingAliasPath;

            string className = DataClassInfoProvider.GetClassName(classId);
            if (!string.IsNullOrEmpty(className))
            {
                selectClassNames.Value = className;
            }

            // Show that the scope was created updated successfully
            if (QueryHelper.GetString("saved", string.Empty) == "1")
            {
                lblInfo.Visible = true;
                lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
            }

            cultureSelector.Value = cultureId;
        }

        string workflowScopes = ResHelper.GetString("Development-Workflow_Scope_New.Scopes");
        string workflowScopesUrl = "~/CMSSiteManager/Development/Workflows/Workflow_Scopes.aspx?workflowid=" + workflowId + "&siteid=" + siteId;
        string currentScope = "";
        if (workflowScopeId > 0)
        {
            currentScope = ResHelper.GetString("Development-Workflow_Scope_New.ScopeID") + workflowScopeId;
        }
        else
        {
            currentScope = ResHelper.GetString("Development-Workflow_Scope_New.Scope");
        }

        // Initialize page title
        InitializeMasterPage(workflowScopes, workflowScopesUrl, currentScope);

        // Hide culture selector due to license restrictions
        plcCulture.Visible = LicenseHelper.CheckFeature(UrlHelper.GetCurrentDomain(), FeatureEnum.Multilingual);
    }

    #endregion


    #region "Other methods"

    /// <summary>
    /// Initializes the master page elements
    /// </summary>
    private void InitializeMasterPage(string workflowScopes, string workflowScopesUrl, string currentScope)
    {
        // Set master page title
        CurrentMaster.Title.HelpTopicName = "newedit_scope";
        CurrentMaster.Title.HelpName = "helpTopic";

        // Initialize breadcrumbs
        string[,] pageTitleTabs = new string[2, 3];
        pageTitleTabs[0, 0] = workflowScopes;
        pageTitleTabs[0, 1] = workflowScopesUrl;
        pageTitleTabs[0, 2] = "workflowsContent";
        pageTitleTabs[1, 0] = currentScope;
        pageTitleTabs[1, 1] = string.Empty;
        pageTitleTabs[1, 2] = string.Empty;

        CurrentMaster.Title.Breadcrumbs = pageTitleTabs;
    }

    #endregion


    #region "Button handling"

    /// <summary>
    /// Saves data of edited workflow scope from TextBoxes into DB.
    /// </summary>
    protected void btnOK_Click(object sender, EventArgs e)
    {
        // finds whether required fields are not empty
        string result = new Validator().NotEmpty(pathElem.Value, ResHelper.GetString("Development-Workflow_Scope_Edit.RequiresStartingAliasPath")).Result;

        string className = ValidationHelper.GetString(selectClassNames.Value, ALL_DOCTYPES);
        int classId = 0;
        if (className != ALL_DOCTYPES)
        {
            // Get class ID
            DataClassInfo dci = DataClassInfoProvider.GetDataClass(className);
            classId = dci.ClassID;
        }

        if (result == string.Empty)
        {
            if (workflowScopeId > 0)
            {
                WorkflowScopeInfo wsi = WorkflowScopeInfoProvider.GetWorkflowScopeInfo(workflowScopeId);
                if (wsi != null)
                {
                    if (!pathElem.Value.ToString().StartsWith("/"))
                    {
                        pathElem.Value = "/" + pathElem.Value;
                    }
                    wsi.ScopeStartingPath = pathElem.Value.ToString().Trim();
                    wsi.ScopeClassID = classId;
                    wsi.ScopeID = workflowScopeId;
                    wsi.ScopeCultureID = ValidationHelper.GetInteger(cultureSelector.Value, 0);
                    WorkflowScopeInfoProvider.SetWorkflowScopeInfo(wsi);
                    lblInfo.Visible = true;
                    lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
                }
            }
            else
            {
                if (workflowId > 0)
                {
                    if (siteId > 0)
                    {
                        if (!pathElem.Value.ToString().StartsWith("/"))
                        {
                            pathElem.Value = "/" + pathElem.Value;
                        }
                        WorkflowScopeInfo wsi = new WorkflowScopeInfo();
                        wsi.ScopeStartingPath = pathElem.Value.ToString().Trim();
                        wsi.ScopeClassID = classId;
                        wsi.ScopeCultureID = ValidationHelper.GetInteger(cultureSelector.Value, 0);
                        wsi.ScopeSiteID = siteId;
                        wsi.ScopeWorkflowID = workflowId;
                        try
                        {
                            WorkflowScopeInfoProvider.SetWorkflowScopeInfo(wsi);
                            UrlHelper.Redirect("Workflow_Scope_Edit.aspx?scopeid=" + wsi.ScopeID + "&saved=1");
                        }
                        catch (Exception ex)
                        {
                            lblError.Visible = true;
                            lblError.Text = ex.Message;
                        }
                    }
                    else
                    {
                        lblError.Visible = true;
                        lblError.Text = ResHelper.GetString("Development-Workflow_Scope_Edit.NoSiteIdGiven");
                    }
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = ResHelper.GetString("Development-Workflow_Scope_Edit.NoDocumentTypeGiven");
                }
            }
        }
        else
        {
            lblError.Visible = true;
            lblError.Text = result;
        }
    }

    #endregion
}
