using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.UI;

using CMS.PortalControls;
using CMS.CMSHelper;
using CMS.GlobalHelper;
using CMS.Controls;
using CMS.Forums;

public partial class CMSWebParts_Forums_ForumMostActiveThreads : CMSAbstractWebPart
{
    #region "Public properties"

    /// <summary>
    /// Gest or sest the cache item name
    /// </summary>
    public override string CacheItemName
    {
        get
        {
            return base.CacheItemName;
        }
        set
        {
            base.CacheItemName = value;
            this.forumDataSource.CacheItemName = value;
        }
    }


    /// <summary>
    /// Cache dependencies, each cache dependency on a new line
    /// </summary>
    public override string CacheDependencies
    {
        get
        {
            return ValidationHelper.GetString(base.CacheDependencies, this.forumDataSource.CacheDependencies);
        }
        set
        {
            base.CacheDependencies = value;
            this.forumDataSource.CacheDependencies = value;
        }
    }


    /// <summary>
    /// Gets or sets the cache minutes
    /// </summary>
    public override int CacheMinutes
    {
        get
        {
            return base.CacheMinutes;
        }
        set
        {
            base.CacheMinutes = value;
            this.forumDataSource.CacheMinutes = value;
        }
    }


    /// <summary>
    /// Gets or sets transformation name
    /// </summary>
    public string TransformationName
    {
        get
        {
        	 return ValidationHelper.GetString(this.GetValue("TransformationName"), ""); 
        }
        set
        {
        	 this.SetValue("TransformationName", value); 
        }
    }


    /// <summary>
    /// Gets or sets forum groups names, use semicolon like separator
    /// </summary>
    public string ForumGroups
    {
        get
        {
        	 return ValidationHelper.GetString(this.GetValue("ForumGroups"), ""); 
        }
        set
        {
        	 this.SetValue("ForumGroups", value); 
        }
    }


    /// <summary>
    /// Gets or sets WHERE condition.
    /// </summary>
    public string WhereCondition
    {
        get
        {
            return ValidationHelper.GetString(this.GetValue("WhereCondition"), "");
        }
        set
        {
            this.SetValue("WhereCondition", value);
            forumDataSource.WhereCondition = value;
        }
    }


    /// <summary>
    /// Gets or sets HideControlForZeroRows property.
    /// </summary>
    public bool HideControlForZeroRows
    {
        get
        {
            return ValidationHelper.GetBoolean(this.GetValue("HideControlForZeroRows"), true);
        }
        set
        {
            this.SetValue("HideControlForZeroRows", value);
            this.repMostActiveThread.HideControlForZeroRows = value;
        }
    }


    /// <summary>
    /// Gets or sets ZeroRowsText property.
    /// </summary>
    public string ZeroRowsText
    {
        get
        {
            return ValidationHelper.GetString(this.GetValue("ZeroRowsText"), "");
        }
        set
        {
            this.SetValue("ZeroRowsText", value);
            this.repMostActiveThread.ZeroRowsText = value;
        }
    }


    /// <summary>
    /// Gets or sets TopN
    /// </summary>
    public int SelectTopN
    {
        get
        {
        	 return ValidationHelper.GetInteger(this.GetValue("SelectTopN"), -1); 
        }
        set
        {
            this.SetValue("SelectTopN", value);
            this.forumDataSource.TopN = value;
        }
    }


    /// <summary>
    /// Gets or sets the site name
    /// </summary>
    public string SiteName
    {
        get
        {
            return DataHelper.GetNotEmpty(this.GetValue("SiteName"), CMSContext.CurrentSiteName);
        }
        set
        {
            this.SetValue("SiteName", value);
            forumDataSource.SiteName = value;
        }
    }

    #endregion


    /// <summary>
    /// Content loaded event handler
    /// </summary>
    public override void OnContentLoaded()
    {
        base.OnContentLoaded();
        SetupControl();
    }
    

    /// <summary>
    /// Reloads the control data
    /// </summary>
    public override void ReloadData()
    {
        base.ReloadData();
        SetupControl();
    }


    /// <summary>
    /// Initializes the control properties
    /// </summary>
    protected void SetupControl()
    {
        if (this.StopProcessing)
        {
            // Do nothing
        }
        else
        {
            if (!String.IsNullOrEmpty(this.TransformationName))
            {
                this.repMostActiveThread.ItemTemplate = CMSDataProperties.LoadTransformation(this, this.TransformationName, false);
                this.repMostActiveThread.HideControlForZeroRows = this.HideControlForZeroRows;
                this.repMostActiveThread.ZeroRowsText = this.ZeroRowsText;

                this.forumDataSource.TopN = this.SelectTopN;
                this.forumDataSource.OrderBy = "PostThreadPosts DESC";
                this.forumDataSource.CacheItemName = this.CacheItemName;
                this.forumDataSource.CacheDependencies = this.CacheDependencies;
                this.forumDataSource.CacheMinutes = this.CacheMinutes;
                this.forumDataSource.FilterName = ValidationHelper.GetString(this.GetValue("WebPartControlID"), this.ClientID);
                this.forumDataSource.SelectOnlyApproved = false;
                this.forumDataSource.SiteName = this.SiteName;

                #region "Complete where condition"

                string where = "";

                // Get groups part of where condition
                string[] groups = this.ForumGroups.Split(';');
                foreach (string group in groups)
                {
                    if (group != "")
                    {
                        if (where != "")
                        {
                            where += " OR ";
                        }
                        where += "(GroupName = N'" + group.Replace("'", "''") + "')";
                    }
                }
                where = "(" + (where == "" ? "(GroupName NOT LIKE 'AdHoc%')" : "(" + where + ")") + " AND (GroupGroupID IS NULL) AND (PostLevel = 0))";

                // Append where condition and set PostLevel to 0 (only threads are needed)
                // and filter out AdHoc forums
                if (!String.IsNullOrEmpty(this.WhereCondition))
                {
                    where += " AND (" + this.WhereCondition + ")";
                }

                #endregion

                this.forumDataSource.WhereCondition = ForumInfoProvider.CombineSecurityWhereCondition(where, 0);
                this.repMostActiveThread.DataSource = this.forumDataSource.DataSource;

                if (!DataHelper.DataSourceIsEmpty(this.repMostActiveThread.DataSource))
                {
                    this.repMostActiveThread.DataBind();
                }
            }
        }
    }


    /// <summary>
    /// OnPreRender override.
    /// </summary>
    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        // Hide control for zero rows
        if (((this.repMostActiveThread.DataSource == null) || (DataHelper.DataSourceIsEmpty(this.repMostActiveThread.DataSource))) && (this.HideControlForZeroRows))
        {
            this.Visible = false;
        }
    }


    /// <summary>
    /// Clears cache.
    /// </summary>
    public override void ClearCache()
    {
        this.forumDataSource.ClearCache();
    }
}
